@@ -1,10 +1,10 @@
 # pylint: disable=unused-argument,invalid-name,line-too-long
-from typing import List, Tuple
+from typing import List, Optional
 
 from parse import parse
 from sqlalchemy import text as sql_text
 
-from alembic_utils.exceptions import FailedToGenerateComparable, SQLParseFailure
+from alembic_utils.exceptions import SQLParseFailure
 from alembic_utils.replaceable_entity import ReplaceableEntity
 
 
@@ -34,6 +34,20 @@ class PGTrigger(ReplaceableEntity):
 
     _template = "create{:s}trigger{:s}{signature}{:s}{event}{:s}ON{:s}{on_entity}{:s}{action}"
 
+    def __init__(
+        self, schema: str, signature: str, definition: str, on_entity: Optional[str] = None
+    ):
+        super().__init__(schema=schema, signature=signature, definition=definition)
+        self._on_entity = on_entity
+
+    @property
+    def identity(self) -> str:
+        """A string that consistently and globally identifies a function
+
+        Overriding default to add the "on table" clause
+        """
+        return f"{self.schema}.{self.signature}-{self.on_entity}"
+
     @classmethod
     def from_sql(cls, sql: str) -> "PGTrigger":
         """Create an instance instance from a SQL string"""
@@ -92,6 +106,9 @@ class PGTrigger(ReplaceableEntity):
     @property
     def on_entity(self) -> str:
         """Get the fully qualified name of the table/view the trigger is applied to"""
+
+        if self._on_entity:
+            return self._on_entity
         create_statement = str(self.to_sql_statement_create())
         result = parse(self._template, create_statement, case_sensitive=False)
         return result["on_entity"]
@@ -103,55 +120,12 @@ class PGTrigger(ReplaceableEntity):
     def to_sql_statement_create_or_replace(self) -> str:
         """ Generates a SQL "create or replace function" statement for PGFunction """
         return f"""
-        {self.to_sql_statement_drop()}
+        DROP TRIGGER IF EXISTS {self.signature} ON {self.on_entity};
         {self.to_sql_statement_create()}
         """
 
-    def get_definition_comparable(self, connection) -> Tuple:
-        """Generates a SQL "create function" statement for PGTrigger
-
-        Had to override create_entity because triggers inherit their schema from
-        the table they're applied to
-        """
-        pg_version_str = connection.execute(sql_text("show server_version_num")).fetchone()[0]
-        pg_version = int(pg_version_str)
-
-        # First try a plain create
-        definition_query = self.get_compare_definition_query()
-
-        try:
-            connection.execute("begin")
-            connection.execute(self.to_sql_statement_create())
-
-            return (self.schema,) + tuple(connection.execute(definition_query).fetchone())
-        except Exception as exc:
-            pass
-        finally:
-            connection.execute("rollback")
-
-        # If  that fails, try a drop and then a create
-        try:
-            connection.execute("begin")
-            connection.execute(self.to_sql_statement_drop())
-            connection.execute(self.to_sql_statement_create())
-            return (self.schema,) + tuple(connection.execute(definition_query).fetchone())
-        except Exception as exc:
-            pass
-        finally:
-            connection.execute("rollback")
-
-        raise FailedToGenerateComparable("Could not simulate entity to get definition comparable")
-
-    def get_identity_comparable(self, connection) -> Tuple:
-        """Generates a SQL "create function" statement for PGTrigger
-
-        Had to override create_entity because triggers inherit their schema from
-        the table they're applied to
-        """
-        return (self.schema, self.identity)
-
     @classmethod
-    def from_database(cls, connection, schema) -> List["PGFunction"]:
+    def from_database(cls, sess, schema) -> List["PGFunction"]:
         """Get a list of all functions defined in the db"""
 
         # NOTE(OR): Schema is looked up by unqualified trigger name. Mismatches possible
@@ -167,11 +141,10 @@ class PGTrigger(ReplaceableEntity):
                     on lower(pgt.tgname) = lower(itr.trigger_name)
         where
             not tgisinternal
-            and itr.event_object_schema = :schema
+            and itr.event_object_schema like :schema
         """
         )
-
-        rows = connection.execute(sql, schema=schema).fetchall()
+        rows = sess.execute(sql, {"schema": schema}).fetchall()
 
         db_triggers = [PGTrigger.from_sql(x[2]) for x in rows]
 
@@ -179,17 +152,3 @@ class PGTrigger(ReplaceableEntity):
             assert trig is not None
 
         return db_triggers
-
-    def get_compare_definition_query(self):
-        return f"""
-        select
-            pg_get_triggerdef(pgt.oid) definition
-        from
-            pg_trigger pgt
-            inner join information_schema.triggers itr
-                on lower(pgt.tgname) = lower(itr.trigger_name)
-        where
-            not tgisinternal
-            and itr.event_object_schema = '{self.schema}'
-            and tgname = '{self.signature}'
-        """
